#ifndef ANDROID_VERSION_H_
#define ANDROID_VERSION_H_

#define ANDROID_VERSION_MAJOR 10
#define ANDROID_VERSION_MINOR 0
#define ANDROID_VERSION_PATCH 0
#define ANDROID_VERSION_PATCH2 0
#define ANDROID_VERSION_PATCH3 0

#endif
