/*
**
** Copyright 2017, The Android Open Source Project
**
** This file is dual licensed.  It may be redistributed and/or modified
** under the terms of the Apache 2.0 License OR version 2 of the GNU
** General Public License.
*/

#pragma once

#ifdef __cplusplus
extern "C" {
#endif

int __android_log_is_debuggable();

#ifdef __cplusplus
}
#endif
